from __future__ import absolute_import, division, print_function, unicode_literals

import subprocess

import os

from publisher import settings
from publisher.classes import Commit
from publisher.utils import WorkingDirectory, get_procedure_code, get_command_output


def get_commits_for_procedure():
    """Get commits for a procedure.
    """
    return get_commits_for_repo()


def get_commits_for_phase(phase_code):
    """Get commits for a phase.
    """

    procedure_code = get_procedure_code()
    return get_commits_for_repo(relative_directory=os.path.join(procedure_code, phase_code))


def get_commits_for_repo(relative_directory=None):
    commit_list = []
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        subprocess.check_output(['git', 'fetch', 'origin', 'refs/notes/*:refs/notes/*'])
        if relative_directory:
            output, error = get_command_output(['git', 'log', '--oneline', '--', relative_directory])
            log_list = output.split("\n")
        else:
            output, error = get_command_output(['git', 'log', '--oneline'])
            log_list = output.split("\n")

        for counter, log in enumerate(log_list):

            if log != '':
                commit_id = log.split()[0]
                comment = log.split(" ", 1)[1]
                note = get_commit_note(commit_id)
                author = get_author(commit_id)

                commit_list.append(Commit(commit_id, comment, author, note))

    return commit_list


def get_author(commit_hash):
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        output, _ = get_command_output(['git', 'show', '--format="%aN <%aE>"', commit_hash])
        return output.split('"')[1]


def get_commit_note(commit_object):
    # Get the notes for the previous commits
    with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):

        try:
            note, error = get_command_output(['git', 'notes', 'show', commit_object])

            if 'no note found' not in error.lower():
                note = note.split('\n')[0]
            else:
                note = None

        except subprocess.CalledProcessError:
            note = None

    return note



from __future__ import absolute_import, division, print_function, unicode_literals

import os

from publisher import settings
from publisher.processing.data_sources.utils import read_csv_file


def get_procedure_details(procedure_code, production_data):

    procedure_row = next((r for r in production_data if is_procedure_row(procedure_code, r)), None)
    if not procedure_row:
        raise EOFError("Could not find Procedure: %s in production csv." % procedure_code)

    procedure_name = procedure_row["Procedure Name"] or "%s - UNKNOWN" % procedure_code
    specialties = procedure_row["Speciality"].split("~") if procedure_row["Speciality"] else []
    channel = procedure_row["Channel"]
    vbs = procedure_row["VBS"]

    return procedure_name, specialties, channel, vbs


def get_procedure_phase_list(procedure_code, production_data):
    return [get_phase_details(r) for r in production_data if is_procedure_row(procedure_code, r)]


def get_phase_data(phase_code, production_data):
    return [get_phase_details(r) for r in production_data if is_phase_row(phase_code, r)]


def get_phase_details(r):
    return {"phase_code": r["Name"], "released_as": r["Released As"], 'phase_name': r['Module Name'],
            'procedure_code': r["Procedure Group"], 'vbs': r["VBS"]}


def is_procedure_row(procedure_code, r):
    return r["Procedure Group"] == procedure_code and r["Status"] != "RETIRED"


def is_phase_row(phase_code, r):
    return r["Name"] == phase_code and r["Status"] != "RETIRED"


def get_production_csv_data():
    if os.path.isfile(settings.BASE_DATA_DIR):
        module_csv = os.path.join(settings.PRODUCTION_INFO_DIR, "module.csv")
    else:
        module_csv = os.path.join(settings.PRODUCTION_INFO_BACKUP_DIR, "module.csv")

    headers, rows = read_csv_file(module_csv)
    return rows



from __future__ import absolute_import, division, print_function, unicode_literals

import datetime
import json

import math

import os

from publisher import settings
from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.csv_steps_file import get_csv_steps_file
from publisher.processing.data_sources.utils import read_csv_file
from publisher.utils import get_procedure_code


def get_step_data(phase_code):
    headers, rows = get_csv_info(phase_code)

    learn_data = []
    test_data = []

    i = datetime.datetime.now()
    info_text = "This phase was built on %s/%s/%s at %s:%s" % (i.day, i.month, i.year, i.hour, i.minute)
    learn_data.append({
        "name": "Build Information",
        "steps": [{
            "stepId": "999999",
            "text": info_text,
            "firstImage": rows[0]["firstImageId"],
            "lastImage": "",
            "type": "Button"
        }],
    })

    for row in rows:
        # Update current objective
        if row["objective"]:
            cur_objective = row["objective"]
            learn_data.append({
                "name": cur_objective,
                "steps": []
            })

            test_data.append({
                "name": cur_objective,
                "steps": []
            })

        # Get all step text into 1 variable
        concat_text = row["text"]
        if "textBoxUpper" in row and row["textBoxUpper"]:
            concat_text += "\n\n" + row["textBoxUpper"]
        if "textBoxLower" in row and row["textBoxLower"]:
            concat_text += "\n\n" + row["textBoxLower"]
        if "moreInfo" in row and row["moreInfo"]:
            concat_text += "\n\n" + row["moreInfo"]
        concat_text = concat_text.replace("\\n", "\n")

        # Add generic step data to dictionary
        learn_data[-1]["steps"].append({
            "stepId": row["stepId"],
            "text": concat_text,
            "firstImage": row["firstImageId"],
            "lastImage": row["lastImageId"],
            "type": row["type"],
        })

        if "interaction_sim2dData" in row and row["interaction_sim2dData"] and not row["interaction_sim2dData"] == "{}":
            interaction_data = json.loads(row["interaction_sim2dData"])
            if interaction_data["type"] == "dragger" and interaction_data["learn"]:
                learn_data[-1]["steps"][-1]["dragger"] = {
                    "start": [interaction_data["p1x"], interaction_data["p1y"]],
                    "target": [interaction_data["p2x"], interaction_data["p2y"]],
                }

            if interaction_data["type"] == "dirDrag":
                a = [interaction_data["p1x"], interaction_data["p1y"]]
                b = [interaction_data["p2x"], interaction_data["p2y"]]

                v1 = (0, 1)
                v2 = (a[0] - b[0], a[1] - b[1])

                v1_theta = math.atan2(v1[1], v1[0])
                v2_theta = math.atan2(v2[1], v2[0])

                # (round to fix yaml serialization error)
                angle = round((v2_theta - v1_theta) * (180.0 / math.pi))

                if angle < 0:
                    angle += 360.0

                # Calculate lengths between points
                # alpha = b[0] - a[0]
                # beta = b[1] - a[1]

                # Get distance between 2 points (round to fix yaml serialization error)
                # distance = round(math.hypot(alpha, beta), 2)
                distance = 60

                learn_data[-1]["steps"][-1]["dirDragger"] = {
                    "start": [interaction_data["p1x"], interaction_data["p1y"]],
                    "tolerance": 20,
                    "angle": angle,
                    "distance": distance,
                }

            if interaction_data["type"] == "tap":
                learn_data[-1]["steps"][-1]["tap"] = {
                    "target": [interaction_data["p1x"], interaction_data["p1y"]],
                }

            if interaction_data["type"] == "hold":
                if "seconds" not in interaction_data:
                    interaction_data["seconds"] = 2

                if "start" not in interaction_data:
                    interaction_data["start"] = interaction_data["seconds"]

                if "finish" not in interaction_data:
                    interaction_data["finish"] = 0

                if "units" not in interaction_data:
                    interaction_data["units"] = "sec"

                learn_data[-1]["steps"][-1]["hold"] = {
                    "target": [interaction_data["p1x"], interaction_data["p1y"]],
                    "duration": int(interaction_data["seconds"]) * 1000,
                    "start": interaction_data["start"],
                    "finish": interaction_data["finish"],
                    "units": interaction_data["units"],
                }

        # Add other sim2d data, infoPoints, labels and pips
        if "infoPoints_sim2dData" in row and row["infoPoints_sim2dData"] and not row["infoPoints_sim2dData"] == "{}":
            info_point_data = json.loads(row["infoPoints_sim2dData"])
            data = {}
            for i, info_point in enumerate(info_point_data):
                data[i] = {
                    "text": info_point_data[info_point]["text"],
                    "targetPosition": info_point_data[info_point]["targetPosition"],
                    "moreInfoId": info_point_data[info_point]["moreInfoId"],
                }
            learn_data[-1]["steps"][-1]["infoPoints"] = info_point_data
        if "pip_sim2dData" in row and row["pip_sim2dData"] and not row["pip_sim2dData"] == "{}":
            pip_data = json.loads(row["pip_sim2dData"])

            # Set the first letter of position to be lower case
            position = pip_data["position"][0].lower() + pip_data["position"][1:]
            if position == "midLeft":
                position = "middleLeft"
            elif position == "midRight":
                position = "middleRight"

            learn_data[-1]["steps"][-1]["pip"] = {
                "position": position,
                "expandable": pip_data["expandable"],
                "shrinkable": pip_data["expandable"],
                "view": pip_data["view"],
                "sequence": pip_data["sequence"],
            }
            if pip_data["sequence"]:
                learn_data[-1]["steps"][-1]["pip"]["video"] = ""

        if "labels_sim2dData" in row and row["labels_sim2dData"] and not row["labels_sim2dData"] == "{}":
            label_data = json.loads(row["labels_sim2dData"])
            data = {}
            for i, label in enumerate(label_data):
                data[i] = {
                    "text": label_data[label]["text"],
                    "textPosition": label_data[label]["textPosition"],
                    "targetPosition": label_data[label]["targetPosition"],
                    "moreInfoId": label_data[label]["moreInfoId"],
                }
            learn_data[-1]["steps"][-1]["labels"] = data

        if "gallery_sim2dData" in row and row["gallery_sim2dData"] and not row["gallery_sim2dData"] == "{}":
            video_ext = [".mp4"]
            img_ext = [".jpg", ".png"]

            gallery_data = json.loads(row["gallery_sim2dData"])
            data = {}

            for i, gallery_item in enumerate(gallery_data):

                main_file = os.path.splitext(os.path.basename(gallery_data[str(i)]["mainGraphic"]))
                thumb_file = os.path.splitext(os.path.basename(gallery_data[str(i)]["thumbnail"]))

                if main_file[1] in video_ext:
                    file_type = "video"
                elif main_file[1] in img_ext:
                    file_type = "image"
                else:
                    raise TypeError("Unrecognised file type: %s" % (main_file[0] + main_file[1]))

                data[i] = {
                    "title": gallery_data[str(i)]["title"],
                    "text": gallery_data[str(i)]["text"],
                    "type": file_type,
                    "mainFile": main_file[0],
                    "thumbFile": thumb_file[0],
                }

            learn_data[-1]["steps"][-1]["gallery"] = data

        if "flipbook_sim2dData" in row and row["flipbook_sim2dData"] and not row["flipbook_sim2dData"] == "{}":
            flip_book_data = json.loads(row["flipbook_sim2dData"])

            video_file = os.path.splitext(os.path.basename(flip_book_data["videoName"]))
            image_file = os.path.splitext(os.path.basename(flip_book_data["imageName"]))

            learn_data[-1]["steps"][-1]["flipbook"] = {
                "image": image_file[0],
                "video": video_file[0],
                "scrubberPos": flip_book_data["scrubberPos"],
            }

        # Store list of information cards needed
        if "moreInfoId" in row and row["moreInfoId"]:
            learn_data[-1]["steps"][-1]["moreInfoId"] = row["moreInfoId"]
            learn_data[-1]["steps"][-1]["moreInfoLabel"] = row["moreInfoLabel"]

        if not row["testable"].lower() == "false":
            mcq_exists = False

            test_data[-1]["steps"].append({
                "stepId": row["stepId"],
                "testText": row["testText"],
                "firstImage": row["firstImageId"],
                "lastImage": row["lastImageId"],
                "type": row["type"]
            })

            if "testFirstImageId" in row and row["testFirstImageId"]:
                test_data[-1]["steps"][-1]["testFirstImage"] = row["testFirstImageId"]

            if "testLastImageId" in row and row["testLastImageId"]:
                test_data[-1]["steps"][-1]["testLastImage"] = row["testLastImageId"]

            if row["testText"]:
                choice_list = []
                if row["choice2"]:
                    choice_list.append(row["choice2"])
                if row["choice3"]:
                    choice_list.append(row["choice3"])
                if row["choice4"]:
                    choice_list.append(row["choice4"])

                if row["choiceCorrect"]:
                    mcq_exists = True
                    test_data[-1]["steps"][-1]["correctChoice"] = row["choiceCorrect"]
                    test_data[-1]["steps"][-1]["incorrectChoices"] = choice_list

            if "interaction_sim2dData" in row and row["interaction_sim2dData"] and not \
               row["interaction_sim2dData"] == "{}":
                if interaction_data["type"] == "dragger" and interaction_data["test"]:
                    if mcq_exists:
                        # Set MCQ last frame to be empty to allow for question dragger combo
                        test_data[-1]["steps"][-1]["lastImage"] = ""

                        test_data[-1]["steps"].append({
                            "stepId": row["stepId"] + "_d",
                            "testText": row["testText"],
                            "firstImage": row["firstImageId"],
                            "lastImage": row["lastImageId"],
                            "type": row["type"],
                        })

                        if "testFirstImageId" in row and row["testFirstImageId"]:
                            test_data[-1]["steps"][-1]["testFirstImage"] = row["testFirstImageId"]

                        if "testLastImageId" in row and row["testLastImageId"]:
                            test_data[-1]["steps"][-1]["testLastImage"] = row["testLastImageId"]

                    test_data[-1]["steps"][-1]["testDragger"] = {
                        "start": [interaction_data["p1x"], interaction_data["p1y"]],
                        "target": [interaction_data["p2x"], interaction_data["p2y"]],
                    }

    return learn_data, test_data


def get_csv_info(phase_code):
    if os.path.isdir(settings.BASE_DATA_DIR):
        csv_file = get_csv_steps_file(phase_code)
    else:
        procedure_code = get_procedure_code()
        csv_file = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, procedure_code, "build_assets", "csv",
                                "%s.csv" % phase_code)

    if csv_file:
        headers, rows = read_csv_file(csv_file)
    else:
        raise ContentMissingError("No csv could be found for phase: %s" % phase_code)

    return headers, rows



from __future__ import absolute_import, division, print_function, unicode_literals

from ruamel.yaml.comments import CommentedMap

from publisher.processing.data_sources.utils import copy_assets
from publisher.processing.data_sources.info_card import get_more_info_data

INFORMATION_CARD_LIST = []


class OverviewWidget:
    def __init__(self, overview_data):
        self.cards = []

        for page in overview_data:
            formatted_page = InformationCardWidget(page)
            self.cards.append(formatted_page.dump()[0])

    def dump(self):
        return self.cards


class InformationCardWidget:
    def __init__(self, info_data):
        self.infoId = info_data["code"]
        self.cta_text = ""
        self.title = info_data["title"]
        self.subTitle = info_data["subTitle"]
        self.assets = []
        self.sections = []

        if "cta_text" in info_data:
            self.cta_text = info_data.get("cta_text", "")

        if "sourceDir" in info_data:
            self.sourceDir = info_data["sourceDir"]

        for section in info_data["sections"]:
            formatted_section = InformationCardSection(section)
            self.sections.append(formatted_section.dump())

    def dump(self):
        return [
            {
                "information_card": CommentedMap([
                    ("code", self.infoId),
                    ("cta_text", self.cta_text),
                    ("title", self.title),
                    ("subTitle", self.subTitle),
                    ("assets", self.assets),
                    ("sections", self.sections)
                ])
            }
        ]


class InformationCardSection:
    def __init__(self, data):
        self.title = data["title"]
        self.body = data["body"]

    def dump(self):
        return CommentedMap([
            ("title", self.title),
            ("body", self.body)
        ])


class Asset:
    def __init__(self, asset_type, step_id="", name=""):
        self.type = asset_type
        self.stepId = step_id
        self.code = ""
        self.name = name

    def dump(self):
        return {
            self.type: CommentedMap([
                ("code", "%s_%s" % (self.stepId, self.type)),
                ("name", self.name)
            ])}


class EulaAsset:
    def __init__(self, eula_file):
        self.eulaFile = eula_file

        if eula_file:
            self.required = "yes"
        else:
            self.required = "no"

    def dump(self):
        return CommentedMap([
            ("eula_asset", self.eulaFile),
            ("requires_complete_profile", self.required)
        ])


class AutoCompleteWidget:
    def __init__(self, step_id):
        self.stepId = step_id

    def dump(self):
        return {
            "auto_complete": CommentedMap([
                ("code", self.stepId + "_autocomplete_1"),
                ("duration", 1300),
            ])
        }


class DraggerWidget:
    def __init__(self, step_id, xy_points):
        self.stepId = step_id
        self.xyPoints = xy_points

    def dump(self):
        return {
            "dragger_ab": CommentedMap([
                ("code", self.stepId + "_dragger_ab_1"),
                ("start_x", float(self.xyPoints["start"][0]) / 320),
                ("start_y", float(self.xyPoints["start"][1]) / 436),
                ("target_x", float(self.xyPoints["target"][0]) / 320),
                ("target_y", float(self.xyPoints["target"][1]) / 436),
            ])
        }


class DirDraggerWidget:
    def __init__(self, step_id, dragger_data):
        self.stepId = step_id
        self.draggerData = dragger_data

    def dump(self):
        return {
            "dragger_directional": CommentedMap([
                ("code", self.stepId + "_dragger_directional_1"),
                ("start_x", float(self.draggerData["start"][0]) / 320),
                ("start_y", float(self.draggerData["start"][1]) / 436),
                ("target_angle", self.draggerData["angle"]),
                ("tolerance", self.draggerData["tolerance"]),
                ("distance", self.draggerData["distance"]),
            ])
        }


class TapWidget:
    def __init__(self, step_id, tap_data):
        self.stepId = step_id
        self.tapData = tap_data

    def dump(self):
        return {
            "tap": CommentedMap([
                ("code", self.stepId + "_tap_1"),
                ("position_x", float(self.tapData["target"][0]) / 320),
                ("position_y", float(self.tapData["target"][1]) / 436),
                ("duration", self.tapData["duration"]),
            ])
        }


class PipWidget:
    def __init__(self, step_id, pip_data, pip_image, pip_video):
        self.stepId = step_id
        self.pipData = pip_data
        self.assets = []

        image_asset = Asset("image", self.stepId, pip_image)
        self.assets.append(image_asset.dump())
        if pip_video:
            video_asset = Asset("video", self.stepId, pip_video)
            self.assets.append(video_asset.dump())

    def dump(self):
        return {
            "pip": CommentedMap([
                ("code", self.stepId + "_pip_1"),
                ("position", self.pipData["position"]),
                ("expandable", self.pipData["expandable"]),
                ("shrinkable", self.pipData["shrinkable"]),
                ("assets", self.assets),
            ])
        }


class LabelWidget:
    def __init__(self, step_id, label_num, label_data, code):
        self.stepId = step_id
        self.labelNum = label_num
        self.labelData = label_data
        self.infoCard = None
        self.parentCode = code

        if not self.labelData["moreInfoId"] == "":
            info_data, source_dir = get_more_info_data(self.labelData["moreInfoId"], self.parentCode)

            if isinstance(info_data, dict):
                info_card = InformationCardWidget(info_data)
                self.infoCard = info_card.dump()
            else:
                self.infoCard = info_data

            INFORMATION_CARD_LIST.append(source_dir)

    def dump(self):
        label = {
            "label": CommentedMap([
                ("code", "%s_label_%d" % (self.stepId, int(self.labelNum) + 1)),
                ("text_anchor_x", float(self.labelData["textPosition"][0]) / 320),
                ("text_anchor_y", float(self.labelData["textPosition"][1]) / 436),
                ("content", self.labelData["text"]),
            ])
        }

        if self.labelData["targetPosition"]:
            label["label"].update({"position_x": float(self.labelData["targetPosition"][0]) / 320})
            label["label"].update({"position_y": float(self.labelData["targetPosition"][1]) / 436})
        else:
            new_x = float(self.labelData["textPosition"][0]) / 320
            new_y = float(self.labelData["textPosition"][1]) / 436

            label["label"].update({"position_x": new_x - 0.001})
            label["label"].update({"position_y": new_y + 0.001})

        if self.infoCard:
            label["label"].update({"information_cards": self.infoCard})

        return label


class InfoPointWidget:
    def __init__(self, step_id, info_point_num, info_point_data, code):
        self.stepId = step_id
        self.infoPointNum = info_point_num
        self.infoPointData = info_point_data
        self.infoCard = None
        self.parentCode = code

        if not self.infoPointData["moreInfoId"] == "":
            info_data, source_dir = get_more_info_data(self.infoPointData["moreInfoId"], self.parentCode)

            if isinstance(info_data, dict):
                info_card = InformationCardWidget(info_data)
                self.infoCard = info_card.dump()
            else:
                self.infoCard = info_data

            INFORMATION_CARD_LIST.append(source_dir)

    def dump(self):
        info_point = {
            "label": CommentedMap([
                ("code", "%s_info_point_%d" % (self.stepId, int(self.infoPointNum) + 1)),
                ("position_x", float(self.infoPointData["targetPosition"][0]) / 320),
                ("position_y", float(self.infoPointData["targetPosition"][1]) / 436),
                ("content", self.infoPointData["text"]),
            ])
        }

        if self.infoCard:
            info_point["label"].update({"information_cards": self.infoCard})

        return info_point


class GalleryWidget:
    def __init__(self, step_id, gallery_data):
        self.stepId = step_id
        self.galleryData = gallery_data
        self.galleryItems = []

    def dump(self):
        for i, item in enumerate(self.galleryData):
            self.galleryItems.append(
                CommentedMap([
                    ("title", self.galleryData[item]["title"]),
                    ("text", self.galleryData[item]["text"]),
                ])
            )

            code = "%s_gallery_%s" % (self.stepId, str(i))
            main_file = Asset(self.galleryData[item]["type"], code, self.galleryData[item]["mainFile"])
            thumb_file = Asset("thumbnail", code, self.galleryData[item]["thumbFile"])

            self.galleryItems[i].update(main_file.dump())
            self.galleryItems[i].update(thumb_file.dump())

        gallery = {
            "gallery": CommentedMap([
                ("code", self.stepId + "_gallery_1"),
                ("items", self.galleryItems),
            ])
        }

        return gallery


class FlipbookWidget:
    def __init__(self, step_id, flipbook_data):
        self.stepId = step_id
        self.flipbookData = flipbook_data
        self.assets = []

        code = "%s_flipbook" % self.stepId
        image_asset = Asset("image", code, self.flipbookData["image"])
        self.assets.append(image_asset.dump())
        video_asset = Asset("video", code, self.flipbookData["video"])
        self.assets.append(video_asset.dump())

    def dump(self):
        return {
            "flip_book": CommentedMap([
                ("code", self.stepId + "_flipbook_1"),
                ("scrubber_position", self.flipbookData["scrubberPos"]),
                ("assets", self.assets),
            ])
        }


class McqWidget:
    def __init__(self, step_id, text, correct, incorrect):
        self.stepId = step_id
        self.text = text
        self.correct = correct
        self.incorrect = incorrect

    def dump(self):
        return {
            "mcq": CommentedMap([
                ("code", "TEST_" + self.stepId + "multi_choice_1"),
                ("content", self.text),
                ("answer", self.correct),
                ("choices", self.incorrect),
            ])
        }


class TextWidget:
    def __init__(self, step_id, text):
        self.stepId = step_id
        self.text = text

    def dump(self):
        return {
            "text": CommentedMap([
                ("code", self.stepId + "_text_1"),
                ("content", self.text),
            ])
        }


class Procedure:
    def __init__(self):
        self.code = ""
        self.name = ""
        self.author = ""
        self.organisation = ""
        self.doiCode = ""
        self.distributionGroups = []
        self.overview = []
        self.devices = []
        self.channel = ""
        self.phases = []
        self.specialties = []
        self.eulaFile = ""
        self.eula = ""
        self.assets = []
        self.formattedOverview = []
        self.formattedDevices = []
        self.vbs = False

    def dump(self, yaml_obj, stream):
        # Check if overview is already formatted
        if isinstance(self.overview, dict):
            self.formattedOverview = OverviewWidget(self.overview).dump()
        else:
            self.formattedOverview = self.overview

        if isinstance(self.devices, dict):
            self.formattedDevices = OverviewWidget(self.devices).dump()
        else:
            self.formattedDevices = self.devices

        icon_asset = Asset("icon")
        icon_asset.stepId = "procedure"
        icon_asset.name = "icon"

        card_asset = Asset("card")
        card_asset.stepId = "procedure"
        card_asset.name = "card"

        self.assets.append(icon_asset.dump())
        self.assets.append(card_asset.dump())

        self.eula = EulaAsset(self.eulaFile).dump()

        yaml_obj.dump(CommentedMap([("name", self.name),
                                    ("overview", self.formattedOverview),
                                    ("devices", self.formattedDevices),
                                    ("channel", self.channel),
                                    ("author", self.author),
                                    ("doi_code", self.doiCode),
                                    ("organisation", self.organisation),
                                    ("authorization", self.eula),
                                    ("distribution_groups", self.distributionGroups),
                                    ("phases", [p.code for p in self.phases]),
                                    ("specialties", self.specialties),
                                    ("assets", self.assets)]), stream)


class Phase:
    def __init__(self):
        self.procedureCode = ""
        self.code = ""
        self.released_as = ""
        self.name = ""
        self.publish = "yes"
        self.supported_app = "touchsurgery"
        self.distributionGroups = []
        self.countryRestriction = ""
        self.assets = []
        self.learnObjectives = []
        self.formattedLearnObjectives = []
        self.formattedTestObjectives = []
        self.testObjectives = []
        self.informationCards = []
        self.translationFiles = []
        self.phaseDir = ""
        self.stepNumbers = True
        self.infoStep = True
        self.vbs = False

    def format_learn_data(self):
        for i, objective in enumerate(self.learnObjectives):
            if objective["name"] == "Build Information" and not self.infoStep:
                # Skip formatting the information step
                continue

            # Format steps
            formatted_steps = []
            for step in objective["steps"]:
                # Format any widgets
                widgets = []

                text = step["text"]
                if self.stepNumbers:
                    # Add step numbers to text for QA purposes
                    text = "Step %s:- %s" % (step["stepId"], step["text"])
                text_widget = TextWidget(step["stepId"], text)
                widgets.append(text_widget.dump())

                if step["type"] == "Auto":
                    auto_widget = AutoCompleteWidget(step["stepId"])
                    widgets.append(auto_widget.dump())

                if "dragger" in step:
                    dragger_widget = DraggerWidget(step["stepId"], step["dragger"])
                    widgets.append(dragger_widget.dump())

                if "dirDragger" in step:
                    dir_dragger_widget = DirDraggerWidget(step["stepId"], step["dirDragger"])
                    widgets.append(dir_dragger_widget.dump())

                if "tap" in step:
                    step["tap"]["duration"] = 0
                    tap_widget = TapWidget(step["stepId"], step["tap"])
                    widgets.append(tap_widget.dump())

                if "hold" in step:
                    hold_widget = TapWidget(step["stepId"], step["hold"])
                    widgets.append(hold_widget.dump())

                if "infoPoints" in step:
                    for infoPoint in step["infoPoints"]:
                        info_point_widget = InfoPointWidget(step["stepId"], infoPoint, step["infoPoints"][infoPoint],
                                                            self.procedureCode)
                        widgets.append(info_point_widget.dump())

                if "pip" in step:
                    if step["pip"]["sequence"]:
                        pip_image = "%s%04d" % (step["pip"]["view"], int(step["firstImage"]))
                        pip_video = "video-%s%04d-%s%04d" % (step["pip"]["view"], int(step["firstImage"]),
                                                             step["pip"]["view"], int(step["lastImage"]))
                    else:
                        pip_image = step["pip"]["view"] + "_image"
                        pip_video = None

                    pip_widget = PipWidget(step["stepId"], step["pip"], pip_image, pip_video)
                    widgets.append(pip_widget.dump())

                if "labels" in step:
                    for label in step["labels"]:
                        label_widget = LabelWidget(step["stepId"], label, step["labels"][label], self.procedureCode)
                        widgets.append(label_widget.dump())

                if "gallery" in step:
                    gallery_widget = GalleryWidget(step["stepId"], step["gallery"])
                    widgets.append(gallery_widget.dump())

                if "flipbook" in step:
                    flipbook_widget = FlipbookWidget(step["stepId"], step["flipbook"])
                    widgets.append(flipbook_widget.dump())

                # Get moreInfoData and add to textWidget
                if "moreInfoId" in step:
                    info_data, source_dir = get_more_info_data(step["moreInfoId"], self.procedureCode)

                    if isinstance(info_data, dict):
                        info_card = InformationCardWidget(info_data)
                        formatted_data = info_card.dump()
                    else:
                        formatted_data = info_data

                    INFORMATION_CARD_LIST.append(source_dir)
                    widgets[0]["text"].update({"information_cards": formatted_data})

                # Format image and video assets
                image_asset = Asset("image", step["stepId"], "image%05d" % int(step["firstImage"]))
                assets = [image_asset.dump()]
                if step["firstImage"] and step["lastImage"] and not step["firstImage"] == step["lastImage"]:
                    name = "video-image%05d-image%05d" % (int(step["firstImage"]), int(step["lastImage"]))
                    video_asset = Asset("video", step["stepId"], name)
                    assets.append(video_asset.dump())

                # Format overall step
                formatted_steps.append(CommentedMap([
                    ("code", int(step["stepId"])),
                    ("widgets", widgets),
                    ("assets", assets),
                ]))

            # Format objectives
            self.formattedLearnObjectives.append(CommentedMap([
                ("name", objective["name"]),
                ("code", i + 1),
                ("steps", formatted_steps)
            ]))

    def format_test_data(self):
        for i, objective in enumerate(self.testObjectives):
            # Format steps
            formatted_steps = []
            if objective["steps"]:
                for step in objective["steps"]:
                    widgets = []

                    if "correctChoice" in step:
                        mcq_widget = McqWidget(step["stepId"], step["testText"], step["correctChoice"],
                                               step["incorrectChoices"])
                        widgets.append(mcq_widget.dump())
                    elif "testDragger" in step:
                        if step["testText"]:
                            text_widget = TextWidget("TEST_" + step["stepId"], step["testText"])
                            widgets.append(text_widget.dump())
                        dragger_widget = DraggerWidget("TEST_" + step["stepId"], step["testDragger"])
                        widgets.append(dragger_widget.dump())
                    elif step["type"] == "Auto":
                        auto_widget = AutoCompleteWidget("TEST_" + step["stepId"])
                        widgets.append(auto_widget.dump())
                    else:
                        auto_widget = AutoCompleteWidget("TEST_" + step["stepId"])
                        widgets.append(auto_widget.dump())

                    # Format image and video assets
                    if "testFirstImage" in step:
                        image_asset = Asset("image", "TEST_" + step["stepId"],
                                            "test_image%05d" % int(step["testFirstImage"]))
                    else:
                        image_asset = Asset("image", "TEST_" + step["stepId"], "image%05d" % int(step["firstImage"]))

                    assets = [image_asset.dump()]

                    video_name = ""
                    if step["firstImage"] and step["lastImage"]:
                        video_name = "video-image%05d-image%05d" % (int(step["firstImage"]), int(step["lastImage"]))
                    elif "testFirstImage" in step and "testLastImage" in step:
                        video_name = "video-test_image%05d-test_image%05d" % (int(step["testFirstImage"]),
                                                                              int(step["testLastImage"]))

                    if video_name:
                        video_asset = Asset("video", "TEST_" + step["stepId"], video_name)
                        assets.append(video_asset.dump())

                    # Format overall step
                    formatted_steps.append(CommentedMap([
                        ("code", "TEST_%s" % str(step["stepId"])),
                        ("widgets", widgets),
                        ("assets", assets),
                    ]))

                # Format objectives
                self.formattedTestObjectives.append(CommentedMap([
                    ("name", objective["name"]),
                    ("code", "TEST_%d" % (i + 1)),
                    ("steps", formatted_steps)
                ]))

    def copy_info_assets(self):
        for sourceDir in INFORMATION_CARD_LIST:
            copy_assets(sourceDir, self.phaseDir)

    def dump(self, yaml_obj, stream):
        global INFORMATION_CARD_LIST
        INFORMATION_CARD_LIST = []

        self.format_learn_data()
        self.format_test_data()

        self.copy_info_assets()

        icon_asset = Asset("icon")
        icon_asset.stepId = "phase"
        icon_asset.name = "icon"

        self.assets.append(icon_asset.dump())

        yaml_obj.dump(
            CommentedMap([("name", self.name), ("publish", self.publish), ("supported_app", self.supported_app),
                          ("distribution_groups", self.distributionGroups),
                          ("country_restriction", self.countryRestriction), ("assets", self.assets),
                          ("objectives", self.formattedLearnObjectives),
                          ("test_objectives", self.formattedTestObjectives)]), stream)



from __future__ import absolute_import, division, print_function, unicode_literals


from Crypto.PublicKey import RSA

import os
import shutil
import subprocess
import sys
import getpass

from publisher import settings
from publisher.touchsurgery import TouchSurgery
from publisher.utils import WorkingDirectory, get_command_output, get_input, get_platform


def setup_users_machine():
    """
    User Setup for publish
        1. Check if git is installed
        2. Setup SSH
        3. Cloning the repo and git pull
    """
    if not git_installed():
        win_msg = 'Please install Github for windows from https://desktop.github.com/ before coming back and ' \
                  'continuing and running setup again.'
        mac_msg = 'Please install Git for Mac from https://git-scm.com/download/mac before running setup again.'

        print("%s" % mac_msg if 'darwin' in get_platform() else win_msg)
        sys.exit(1)

    print("Configuring ssh config file")
    check_and_create_directory(settings.SSH_DIRECTORY_PATH)
    check_and_create_file(settings.SSH_CONFIG_PATH)
    if not has_private_key():
        raise RuntimeError("Unable to proceed without a key. Please contact Hansel before trying again")
    configure_ssh_config()

    print('Configuring user profile')
    get_username()
    email = get_email()
    password = get_password()

    print('Generating key')
    sys.stdout.flush()
    pub_key = create_rsa_key_pair()
    login(email, password, pub_key)

    print("Installing and configuring git lfs")
    check_brew_installed()
    install_git_lfs()
    configure_git_lfs()

    check_and_create_directory(settings.GIT_DIRECTORY)

    print("Setting up the content repo")
    clone_content_repo()
    pull_content_repo()

    print("Setting up the channel repo")
    clone_channel_repo()
    pull_channel_repo()


def git_installed():
    try:
        subprocess.check_output(['git', '--version'])
    except OSError:
        print("Git not installed")
        return False
    return True


def get_username():
    print("Please enter your name here:")
    sys.stdout.flush()
    username = get_input()
    subprocess.check_output(['git', 'config', '--global', 'user.name', username])


def get_email():
    print("Please enter your touch surgery email here:")
    sys.stdout.flush()
    email = get_input()
    subprocess.check_output(['git', 'config', '--global', 'user.email', email])
    return email


def get_password():
    print("Please enter your touch surgery password here:")
    sys.stdout.flush()
    if get_platform().startswith('win'):
        print('Your password will not be hidden, so make sure you hide your screen while entering.')
        sys.stdout.flush()
        password = get_input()
    else:
        password = getpass.getpass()
    return password


def has_private_key():
    """Check whether the user has the correct private key
    """
    return os.path.exists(os.path.expanduser("~/.ssh/touchsurgery-studio.pem"))


def configure_ssh_config():
    """Creates and sets up an ssh config file, or appends the necessary entry to an existing one
    """
    shutil.copyfile(os.path.expanduser(settings.SSH_CONFIG_PATH), os.path.expanduser(settings.SSH_CONFIG_PATH + '.bak'))

    obsolete_stanza = (
        'Host studio-git.touchsurgery.com\n '
        'User ubuntu\n '
        'IdentitiesOnly true\n '
        'IdentityFile ~/.ssh/touchsurgery-studio.pem\n'
    )

    ssh_config_stanza = (
        'Host studio-git.touchsurgery.com\n'
        ' User git\n'
        ' IdentitiesOnly true\n'
        ' IdentityFile ' + settings.RSA_PRIVATE_KEY_PATH + '\n'
    )

    try:

        with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "r") as config_file:
            current_config_text = config_file.read()
        ssh_config_missing = ssh_config_stanza not in current_config_text
        obsolete_stanza_present = obsolete_stanza in current_config_text

        # Remove outdated config info
        if obsolete_stanza_present:
            current_config_text = current_config_text.replace(obsolete_stanza, '')
            with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "w") as config_file:
                config_file.write(current_config_text)

        # Add relevant config info
        if ssh_config_missing:
            with open(os.path.expanduser(settings.SSH_CONFIG_PATH), "a+") as config_file:
                config_file.write('\n' + '\n' + ssh_config_stanza)

    except Exception:
        print("Unable to configure the ssh config")
        raise


def check_brew_installed():
    """ Get Macs ready to brew
    """
    if 'darwin' in get_platform():
        output, _ = get_command_output(['brew', 'help'])
        if 'usage' not in output.lower():
            raise Exception("Please install Brew from here: https://brew.sh/")


def install_git_lfs():
    """Install git lfs
    """
    if 'darwin' in get_platform():
        output, _ = get_command_output(['which', 'git-lfs'])
        if 'usr' not in output.lower():
            call_command_and_print_exception(['brew', 'install', 'git-lfs'], "brew lfs install failure")

    call_command_and_print_exception(['git', 'lfs', 'install'], "lfs install failure")


def clone_content_repo():
    """Clone the content repo
    """
    if not os.path.exists(settings.PROCEDURE_CHECKOUT_DIRECTORY):
        try:
            with WorkingDirectory(settings.GIT_DIRECTORY):
                call_command_and_print_exception(['git', 'lfs', 'clone',
                                                  settings.PROCEDURE_REPOSITORY,
                                                  settings.PROCEDURE_CHECKOUT_DIRECTORY], "Clone repo failure")
        except Exception as e:
            print("Unable to clone the content repo")
            raise e
    else:
        print("Procedure repo already exists")


def pull_content_repo():
    """Run a git pull in the content repo
    """
    try:
        with WorkingDirectory(settings.PROCEDURE_CHECKOUT_DIRECTORY):
            call_command_and_print_exception(['git', 'lfs', 'pull', 'origin', 'master'], "Git pull failure")
    except Exception as e:
        print("Unable to run a git pull in the content repo")
        raise e


def clone_channel_repo():
    """Clone the channel repo
    """
    if not os.path.exists(settings.CHANNELS_CHECKOUT_DIRECTORY):
        try:
            with WorkingDirectory(settings.GIT_DIRECTORY):
                call_command_and_print_exception(['git', 'lfs', 'clone',
                                                  settings.CHANNELS_REPOSITORY,
                                                  settings.CHANNELS_CHECKOUT_DIRECTORY], "Clone repo failure")
        except Exception as e:
            print("Unable to clone the content repo")
            raise e
    else:
        print("Channel repo already exists")


def pull_channel_repo():
    """Run a git pull in the channel repo
    """
    try:
        with WorkingDirectory(settings.CHANNELS_CHECKOUT_DIRECTORY):
            call_command_and_print_exception(['git', 'lfs', 'pull', 'origin', 'master'], "Git pull failure")
    except Exception as e:
        print("Unable to run a git pull in the channel repo")
        raise e


def call_command_and_print_exception(command, message):
    try:
        return subprocess.check_output(command)
    except Exception as e:
        print(message)
        raise e


def check_and_create_directory(path):
    try:
        if not os.path.exists(path):
            os.mkdir(path)
    except Exception:
        print("Could not find or create the directory")
        raise


def check_and_create_file(path):
    try:
        if not os.path.exists(path):
            subprocess.check_output(['touch', path])
    except Exception:
        print("Could not find or create the file")
        raise


def configure_git_lfs():
    """Set relevant  lfs settings
    """
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.url',
                                      'https://live.touchsurgery.com/api/v3/lfs'], "lfs config failure")
    call_command_and_print_exception(['git', 'config', '--global', 'lfs.activitytimeout', '60'], "lfs config failure")


def create_rsa_key_pair():
    private_key = settings.RSA_PRIVATE_KEY_PATH
    public_key = settings.RSA_PUBLIC_KEY_PATH

    key = RSA.generate(2048)
    if os.path.exists(private_key):
        os.chmod(private_key, 0o0600)

    with open(private_key, 'wb') as rsa_pri:
        os.chmod(private_key, 0o0600)
        rsa_pri.write(key.exportKey('PEM'))

    pubkey = key.publickey()
    with open(public_key, 'wb') as rsa_pub:
        pub_key = pubkey.exportKey('OpenSSH')
        rsa_pub.write(pubkey.exportKey('OpenSSH'))

    return pub_key.split()[1]


def login(email, password, pub_key):
    """Verify TouchSurgery user here with rsa key and TouchSurgery login
    """
    login_instance = TouchSurgery()
    login_instance.login(email, password)
    login_instance.upload_key(pub_key)



from __future__ import absolute_import, division, print_function, unicode_literals

from abc import abstractmethod

import os

from ruamel.yaml import YAMLError, safe_load

from publisher import settings
from publisher.exceptions import SingleProceduresNotFoundError, MissingYamlFileError, ValidationError, \
    CombinedValidationError, MissingAssetDirectoryError, InvalidYamlFileError
from publisher.utils import get_content_directories


class Validator(object):

    def __init__(self, working_directory):
        self.working_directory = working_directory

    @abstractmethod
    def validate(self):
        pass


class SingleProcedureValidator(Validator):

    def validate(self):
        procedure_directories = get_content_directories(self.working_directory)

        if len(procedure_directories) != 1:
            raise SingleProceduresNotFoundError()


class YamlFileValidator(Validator):

    def __init__(self, yaml_file):
        self.yaml_file = yaml_file

    def validate(self):
        if not os.path.exists(self.yaml_file):
            raise MissingYamlFileError(self.yaml_file)

        try:
            with open(self.yaml_file, 'rb') as y:
                safe_load(y)
        except YAMLError:
            raise InvalidYamlFileError(self.yaml_file)


class ProcedureYamlFileValidator(YamlFileValidator):

    def __init__(self, procedure_directory):
        self.yaml_file = os.path.join(procedure_directory, 'procedure.yml')


class PhaseYamlFileValidator(YamlFileValidator):

    def __init__(self, phase_directory):
        self.yaml_file = os.path.join(phase_directory, 'phase.yml')


class AssetDirectoryValidator(Validator):

    def __init__(self, base_directory):
        self.asset_directory = os.path.join(base_directory, 'assets')

    def validate(self):
        if not os.path.exists(self.asset_directory):
            raise MissingAssetDirectoryError(self.asset_directory)


def validate_procedure_directory():
    SingleProcedureValidator(settings.PROCEDURE_CHECKOUT_DIRECTORY).validate()


def validate_save():

    validate_procedure_directory()

    procedure_directory = os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY,
                                       get_content_directories(settings.PROCEDURE_CHECKOUT_DIRECTORY)[0])

    validators = [ProcedureYamlFileValidator(procedure_directory), AssetDirectoryValidator(procedure_directory)]

    phase_directories = [os.path.join(procedure_directory, d) for d in get_content_directories(procedure_directory)]

    validators += [PhaseYamlFileValidator(d) for d in phase_directories]
    validators += [AssetDirectoryValidator(d) for d in phase_directories]

    errors = []

    for v in validators:
        try:
            v.validate()
        except ValidationError as e:
            errors.append(e)

    if len(errors) > 0:
        raise CombinedValidationError(errors)



from __future__ import absolute_import, division, print_function, unicode_literals

import os

from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.utils import YamlObject
from publisher import settings


def get_organisation_from_channel(channel):
    channel = channel.replace("channel-", "")

    channel_file = os.path.join(settings.CHANNELS_INFO_DIR, channel + ".yml")

    if not os.path.exists(channel_file):
        raise ContentMissingError('Yaml file for channel {0} is missing'.format(channel))

    yaml = YamlObject()

    channel_file = open(channel_file, "r")
    channel_data = yaml.load(channel_file)

    return channel_data["name"]



from __future__ import absolute_import, division, print_function, unicode_literals

import os
from shutil import copyfile

from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.utils import copy_assets
from publisher import settings


def update_thumbnails(procedure, asset_directory):
    print("-- Getting updated icon for procedure --")
    source_asset_dir = _get_thumbnail(procedure.code, procedure.vbs)
    copy_assets(source_asset_dir, asset_directory)


def update_phase_thumbnails(phase, asset_directory):
    phase_icon = os.path.join(asset_directory, "icon.jpg")

    print("-- Getting updated icon for phase --")
    thumbnail_directory = _get_thumbnail(phase.code, phase.vbs)
    thumbnail = os.path.join(thumbnail_directory, "icon.jpg")
    copyfile(thumbnail, phase_icon)


def _get_thumbnail(item_code, vbs=False):
    if not vbs:
        thumbnail_dir = os.path.join(settings.CG_APP5_ROOT, item_code, "thumbnails")
    else:
        thumbnail_dir = os.path.join(settings.VBS_ROOT, item_code, "thumbnails")

    if not os.path.isdir(thumbnail_dir):
        raise ContentMissingError('Expected thumbnail to exist for {0}'.format(item_code))

    return thumbnail_dir



from __future__ import absolute_import, division, print_function, unicode_literals

import subprocess
import os.path
import sys
from shutil import copyfile
import json
from PIL import Image

from publisher.exceptions import ContentMissingError
from publisher.processing.data_sources.step import get_csv_info
from publisher.processing.data_sources.utils import delete_misc_files, copy_assets
from publisher import settings

ffmpegExe = 'ffmpeg'

ffmpegDefaults = [ffmpegExe,
                  # Input format
                  '-f', 'image2',
                  # Input framerate
                  '-r', '25',
                  # Insert position for input frames to be overwritten
                  'INPUT_SETTINGS',
                  # Video codec for the output
                  '-codec:v', 'libx264',
                  # Disables some advanced features but provides for better compatibility with iOS/Android
                  '-profile:v', 'baseline',
                  # Must set colorspace format compatible with x264
                  '-pix_fmt', 'yuv420p',
                  # H.264 level setting
                  '-level:v', '2.2',
                  # Disable CABAC
                  '-coder', '0',
                  # Video filter graph
                  'SCALE_SETTINGS',
                  # Output framerate
                  '-r', '25',
                  # Frequency of keyframes
                  # '-g', '2',
                  # Logging level
                  '-loglevel', 'panic']


def is_ffmpeg_installed():
    p = subprocess.Popen([ffmpegExe, '-h'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.communicate()
    rt = p.poll()
    return rt == 0


def get_pip_dir(module_dir):
    return os.path.join(module_dir, "pipSource")


def get_widget_dir(module_dir):
    return os.path.join(module_dir, "widgetGraphics")


def get_graphics_dir(module_dir):
    asset_dir = os.path.join(module_dir, "graphics")
    return get_latest_version_directory(asset_dir)


def get_phase_directory(phase_code, vbs):
    if not vbs:
        dir_name = os.path.join(settings.CG_APP5_ROOT, phase_code)
    else:
        dir_name = os.path.join(settings.VBS_ROOT, phase_code)

    if not os.path.isdir(dir_name):
        raise ContentMissingError('Source folder not found for %s at %s' % (phase_code, dir_name))

    return dir_name


def get_latest_version_directory(dir_name):
    path, dirs, files = next(os.walk(dir_name))

    if len(dirs) == 0:
        raise EOFError("No asset versions found for: %s" % dir_name)

    return os.path.join(path, dirs[-1])


def convert_files(phase, phase_directory, update_graphics=True, update_pip_graphics=True, update_widget_graphics=True,
                  vbs=False):
    if not is_ffmpeg_installed():
        print("Error: no ffmpeg executable found at: ", ffmpegExe)
        sys.exit(2)

    latest_phase_directory = get_phase_directory(phase.code, vbs)

    if update_graphics:
        graphics_dir = get_graphics_dir(latest_phase_directory)
        graphics_files = get_file_list(graphics_dir)

        if not graphics_files:
            raise ContentMissingError('Source frames not found for {0}'.format(phase.code))

        copy_graphics_files(phase.code, graphics_dir, phase_directory)

    pip_directory = get_pip_dir(latest_phase_directory)
    if update_pip_graphics and os.path.isdir(pip_directory):
        copy_pip_files(phase.code, pip_directory, phase_directory)

    widget_directory = get_widget_dir(latest_phase_directory)
    if update_widget_graphics and os.path.isdir(widget_directory):
        copy_widget_files(widget_directory, phase_directory)


def copy_graphics_files(phase_code, source_directory, dest_dir):
    file_list = get_file_list(source_directory)

    headers, rows = get_csv_info(phase_code)

    frame_numbers = []
    test_frame_numbers = []
    missing_frames = []
    prefix = "image"
    test_prefix = "test_image"

    for row in rows:
        first_frame = int(row["firstImageId"])

        if row["lastImageId"]:
            last_frame = int(row["lastImageId"])
        else:
            last_frame = first_frame

        frame_numbers.append([first_frame, last_frame])

        for i in range(first_frame, last_frame + 1):
            image_name = "%s%05d.jpg" % (prefix, i)

            # Check for missing frames in step
            if image_name not in file_list:
                missing_frames.append(image_name)

        if "testFirstImageId" in row and row["testFirstImageId"]:
            test_first_frame = int(row["testFirstImageId"])

            if "testLastImageId" in row and row["testLastImageId"]:
                test_last_frame = int(row["testLastImageId"])
            else:
                test_last_frame = test_first_frame

            test_frame_numbers.append([test_first_frame, test_last_frame])

            for i in range(test_first_frame, test_last_frame + 1):
                image_name = "%s%05d.jpg" % (test_prefix, i)

                # Check for missing frames in step
                if image_name not in file_list:
                    missing_frames.append(image_name)

    # Error out if missing frames were found
    if missing_frames:
        raise EOFError("Missing frames: %s" % str(missing_frames))

    print("\n-- Converting image files --")
    copy_files(frame_numbers, source_directory, dest_dir, prefix, "%05d")
    copy_files(test_frame_numbers, source_directory, dest_dir, test_prefix, "%05d")


def copy_pip_files(phase_code, source_directory, dest_directory):
    path, dirs, files = next(os.walk(source_directory))

    pip_folders = {}

    for pip_name in dirs:
        pip_folder = os.path.join(path, pip_name)
        pip_version_folder = get_latest_version_directory(pip_folder)
        file_list = get_file_list(pip_version_folder)

        pip_folders[pip_name] = {
            "folder": pip_version_folder,
            "files": file_list
        }

    headers, rows = get_csv_info(phase_code)

    if "pip_sim2dData" not in headers:
        return

    frame_numbers = {}
    missing_frames = {}

    for row in rows:

        pip_info = row["pip_sim2dData"]

        if pip_info and not pip_info == "{}":
            pip_load = json.loads(pip_info)
            pip_name = pip_load["view"]

            if pip_load["sequence"]:
                first_frame = int(row["firstImageId"])
                if row["lastImageId"]:
                    last_frame = int(row["lastImageId"])
                else:
                    last_frame = first_frame

                if pip_name not in frame_numbers:
                    frame_numbers[pip_name] = []

                frame_numbers[pip_name].append([first_frame, last_frame])

                for i in range(first_frame, last_frame):
                    image_name = "%s%04d.jpg" % (pip_name, i)

                    # Check for missing frames in step
                    if image_name not in pip_folders[pip_name]["files"]:
                        if pip_name not in missing_frames:
                            missing_frames[pip_name] = []

                        missing_frames[pip_name].append(image_name)

                # Error out if missing frames were found
                if missing_frames:
                    raise EOFError("Missing frames: %s" % str(missing_frames))

                print("\n-- Converting pip files step: %s --" % row["stepId"])
                for pip in frame_numbers:
                    copy_files(frame_numbers[pip], pip_folders[pip]["folder"], dest_directory, pip, "%04d")

            else:
                image_name = "%s_image%s" % (pip_name, pip_load["ext"])
                pip_files = pip_folders[pip_name]["files"]

                if len(pip_files) > 1:
                    raise EOFError("Too many files for non sequence pip: %s" % str(pip_folders[pip_name]["folder"]))
                elif len(pip_files) < 1:
                    raise EOFError("Missing pip images in: %s" % str(pip_folders[pip_name]["folder"]))

                asset_dir = os.path.join(dest_directory, "assets")
                dest_file = os.path.join(asset_dir, image_name)
                source_file = os.path.join(pip_folders[pip_name]["folder"], pip_files[0])

                copyfile(source_file, dest_file)


def copy_widget_files(source_directory, dest_directory):
    print("\n-- Copying widget files --")
    asset_dir = os.path.join(dest_directory, "assets")
    copy_assets(source_directory, asset_dir)


def copy_files(frame_numbers, source_dir, dest_dir, prefix, padding):
    asset_dir = os.path.join(dest_dir, "assets")

    for frames in frame_numbers:
        # Copy first and last image
        for frame in frames:
            frame_number = padding % frame
            frame_file = "%s%s.jpg" % (prefix, frame_number)

            source_file = os.path.join(source_dir, frame_file)
            dest_file = os.path.join(asset_dir, frame_file)

            copyfile(source_file, dest_file)

        # Convert sequence to video file
        if not frames[1] == frames[0]:
            frame_number_start = padding % frames[0]
            frame_number_end = padding % frames[1]
            video_file_name = "video-%s%s-%s%s.mp4" % (prefix, frame_number_start, prefix, frame_number_end)
            input_name = prefix + "%s.jpg" % padding
            input_image_path = os.path.join(source_dir, input_name)
            output_video_path = os.path.join(asset_dir, video_file_name)
            frame_count = str(frames[1] - frames[0] + 1)

            ffmpeg_options = list(ffmpegDefaults)

            # Insert settings in list - FFMPEG requires a specific order to the arguments
            input_index = ffmpeg_options.index("INPUT_SETTINGS")
            ffmpeg_options[input_index:input_index + 1] = [
                # Set starting frame for image sequence
                "-start_number", str(frames[0]),
                # Force overwrite
                "-y",
                # Add input file range in image0000... format
                "-i", input_image_path,
                # Set frame count for video
                "-frames:v", frame_count]

            # Get resolution of source file if pip
            if prefix != "image":
                image_name = "%s%d.jpg" % (prefix, frames[0])
                image_path = os.path.join(source_dir, image_name)
                im = Image.open(image_path)
                width, height = im.size
                # Set the video scale
                scale_settings = "scale='%d:%d'" % (width, height)
            else:
                # Set the video scale
                scale_settings = "scale='680:1024'"

            scale_index = ffmpeg_options.index("SCALE_SETTINGS")
            ffmpeg_options[scale_index:scale_index + 1] = [
                # Filtergraph option (lets you set scale, pixel aspect, etc.)
                "-vf",
                scale_settings]

            # Set output video location
            ffmpeg_options.append(output_video_path)

            subprocess.call(ffmpeg_options)

            if not os.path.exists(output_video_path):
                raise Exception("Failed to create %s" % output_video_path)

        delete_misc_files(asset_dir)


def get_file_list(directory_name):
    path, dirs, files = next(os.walk(directory_name))

    # Remove thumbnail file from list
    if "Thumbs.db" in files:
        files.remove("Thumbs.db")

    return files



from __future__ import absolute_import, division, print_function, unicode_literals

import sqlite3

from publisher import settings


def get_csv_steps_file(phase_code):
    c = sqlite3.connect(settings.CONTENT_DB_DIR)
    cursor = c.execute("""
        SELECT f.file_path FROM assetdb_asset a
        INNER JOIN assetdb_assetversion v ON a.id = v.asset_id
        INNER JOIN assetdb_assetfile f ON v.id = f.asset_version_id
        WHERE
            a.category = 'Admin' AND
            a."name" = 'Main' AND
            a."level_of_detail" = 'default' AND
            a."variant" = 'default' AND
            a."module" = ? AND
            a."asset_type" = 'csvSteps' AND
            a."department" = 'default' AND
            a."operation" IS NULL AND
            a."institution" = 'TouchSurgeryContent' AND
            a."stage" = 'default' AND
            NOT a.deprecated AND
            NOT v."pending" AND
            NOT v."deprecated" AND
            NOT f."pending" AND
            NOT f."deprecated" AND
            f."name" = 'default'
        ORDER BY v."version" DESC LIMIT 1;""", (phase_code,))

    row = cursor.fetchone()
    c.close()

    return _get_platform_specific_path(row[0].strip()) if row else None


def _get_platform_specific_path(file_path):
    if settings.ORIGINAL_CONTENT_ROOT in file_path:
        file_path = file_path.replace(settings.ORIGINAL_CONTENT_ROOT, settings.REPLACEMENT_CONTENT_ROOT, 1)
    else:
        file_path = file_path.replace(settings.ORIGINAL_CONTENT_ROOT_OLD, settings.REPLACEMENT_CONTENT_ROOT_OLD, 1)

    return file_path.replace("\\", "/")



import os

from publisher import settings


class ProcedureFileManager(object):

    def __init__(self, procedure):
        self._procedure_code = procedure.code
        self._ensure_structure_exists()

        self.phase_files = [PhaseFileManager(p, self._procedure_code) for p in procedure.phases]

    def _ensure_structure_exists(self):
        for d in [self.base_directory, self.asset_directory, self.build_asset_directory, self.more_info_directory,
                  self.overview_directory, self.devices_directory, self.eula_directory, self.csv_directory]:
            if not os.path.exists(d):
                os.makedirs(d)

    @property
    def base_directory(self):
        return os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, self._procedure_code)

    @property
    def procedure_file(self):
        return os.path.join(self.base_directory, 'procedure.yml')

    @property
    def asset_directory(self):
        return os.path.join(self.base_directory, 'assets')

    @property
    def build_asset_directory(self):
        return os.path.join(self.base_directory, 'build_assets')

    @property
    def more_info_directory(self):
        return os.path.join(self.build_asset_directory, 'moreInfo')

    @property
    def overview_directory(self):
        return os.path.join(self.build_asset_directory, 'overview')

    @property
    def devices_directory(self):
        return os.path.join(self.build_asset_directory, 'devices')

    @property
    def eula_directory(self):
        return os.path.join(self.build_asset_directory, 'eula')

    @property
    def csv_directory(self):
        return os.path.join(self.build_asset_directory, 'csv')


class PhaseFileManager(object):

    def __init__(self, phase, procedure_code):
        self._procedure_code = procedure_code
        self.phase_code = phase.code

        self._ensure_structure_exists()

    def _ensure_structure_exists(self):
        for d in [self.base_directory, self.asset_directory, self.translation_directory]:
            if not os.path.exists(d):
                os.makedirs(d)

    @property
    def base_directory(self):
        return os.path.join(self.procedure_directory, self.phase_code)

    @property
    def procedure_directory(self):
        return os.path.join(settings.PROCEDURE_CHECKOUT_DIRECTORY, self._procedure_code)

    @property
    def phase_file(self):
        return os.path.join(self.base_directory, 'phase.yml')

    @property
    def asset_directory(self):
        return os.path.join(self.base_directory, 'assets')

    @property
    def translation_directory(self):
        return os.path.join(self.base_directory, 'translations')



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


setup(
    name='tspublisher',
    version='0.0.1.dev760',
    license='GNU General Public License',
    description='Touch Surgery publishing tools',
    long_description=read('README.md'),
    author='TS Content Squad',
    author_email='admin@touchsurgery.com',
    url='https://www.touchsurgery.com/jobs',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    zip_safe=False,
    keywords=[],
    install_requires=[
        'requests',
        'ruamel.yaml',
        'gitpython',
        'pycryptodome',
        'pillow',
        'future'
    ],
    extras_require={
        'development': [
            'codecov',
            'flake8',
            'mock',
            'pytest',
            'pytest-cov',
            'pytest-pep8',
        ]
    },
    entry_points={
        'console_scripts': [
            'tspub = publisher.cli:main',
        ]
    }
)



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import platform

GIT_DIRECTORY = os.path.expanduser("~/git")
PROCEDURE_CHECKOUT_DIRECTORY = os.path.expanduser("~/git/content-procedures")
CHANNELS_CHECKOUT_DIRECTORY = os.path.expanduser("~/git/content-channels")
STUDIO_GIT_PATH = 'studio-git.touchsurgery.com'
PROCEDURE_REPOSITORY = 'studio-git.touchsurgery.com:/srv/git/procedure-repo'
CHANNELS_REPOSITORY = 'studio-git.touchsurgery.com:/srv/git/channel-repo'

SSH_DIRECTORY_PATH = os.path.expanduser('~/.ssh')
SSH_CONFIG_PATH = os.path.expanduser("~/.ssh/config")
RSA_PUBLIC_KEY_PATH = os.path.expanduser('~/.ssh/touchsurgery_publisher_rsa.pub')
RSA_PRIVATE_KEY_PATH = os.path.expanduser('~/.ssh/touchsurgery_publisher_rsa')

if platform.system() == "Windows":
    BASE_DATA_DIR = "Z:\\"
else:
    BASE_DATA_DIR = os.path.join("/Volumes", "content")

PRODUCTION_INFO_DIR = os.path.join(BASE_DATA_DIR, "productionInfo")
PRODUCTION_INFO_BACKUP_DIR = os.path.expanduser("~/git/production-info")
CHANNELS_INFO_DIR = CHANNELS_CHECKOUT_DIRECTORY

CONTENT_DB_DIR = os.path.join(BASE_DATA_DIR, "assetdb", ".db", "contentdb.sqlite3")

ORIGINAL_CONTENT_ROOT = "C:/TouchSurgery/assetdb/vault2"
ORIGINAL_CONTENT_ROOT_OLD = "C:/TouchSurgery/assetdb"
if platform.system() == "Windows":
    REPLACEMENT_CONTENT_ROOT = "C:/TouchSurgery/assetdb/vault2"
    REPLACEMENT_CONTENT_ROOT_OLD = "C:/TouchSurgery/assetdb"
else:
    REPLACEMENT_CONTENT_ROOT = "/Volumes/content/assetdb/vault2"
    REPLACEMENT_CONTENT_ROOT_OLD = "/Volumes/content/assetdb"

CG_APP5_ROOT = os.path.join(BASE_DATA_DIR, "delivery", "app5")
VBS_ROOT = os.path.join(BASE_DATA_DIR, "delivery", "vbs")
VBS_BACKUP_ROOT = PROCEDURE_CHECKOUT_DIRECTORY
